# GeoModel Kernel Classes

GeoModel Kernel classes provide a set of geometrical primitives for describing detectors, and a scheme for accessing both the raw geometry of a detector and arbitrary detector subsystem-specific geometrical services.  The scheme provides a means of keeping the geometrical services synched to the raw geometry, while incorporating time-dependent alignments.

The design of these classes reflects the belief that raw geometry is highly constrained by the simulation engines, while the readout geometry is highly detector subsystem specific and has practically no constraint at all. Thus, the simulation engines available today (e.g. Geant4), which fortunately have a high degree of conceptual commonality, basically determine the format of the raw geometry description. Every detector subsystem engineer who is responsible for describing a subdetector needs to provide one or more trees of raw geometry for the purpose of simulation. The geometry kernel classes provide a set of geometrical primitives to support the operations of creating and accessing raw geometry.

In addition, the subsystem engineer has to layer, upon this raw geometry, any detector specific readout services required in simulation, reconstruction, or analysis. This is a very broad task and relies heavily on the creativity and intelligence of the subsystem specialist. The specialist is asked only to provide access to this type of information through the same class (e.g. `GeoVDetectorManager`) that accesses the raw geometry.

The set of specialized routines (e.g. GeoModel detector plugins, detector factories) are then all called upon during the initialization phase to build both raw and readout geometries. During normal execution, messages to move various pieces of material to new, aligned positions will be routed from the calibration database to the detector managers which must respond by applying new alignment transformations at specific points in the geometry tree designated as alignable. The position of one or more pieces of raw geometry moves about when the alignable transformations are tweaked.

Readout geometry synchronizes itself to raw geometry by holding a pointer to a volume in the raw geometry tree that holds its absolute transformation with respect to world coordinates in cache. The readout geometry should access this information when responding to any queries about, or relying upon, its absolute position.

{!components/kernel/overview/Overview.md!}